#![no_std]
#![allow(dead_code)]

#[macro_use]
extern crate klee;
use core::ptr;

//# FUNCTION codgen(): UNSIGNED INTEGER;
// #  LOCAL SIGNED INTEGER   n;
// #  LOCAL UNSIGNED INTEGER x, y;
// #  BEGIN
// #    n := [count the number of 0's in word "seed"];
// #    x := [rotate "seed" left by 30 bits];
// #    y := [shift "seed" right-ARITHMETIC by 6 bits];
// #    seed := x XOR y XOR n;
// #   RETURN( seed XOR 0x464b713e );
// #  END;
// #
// # hint:  if "seed" is initialized to 0x3e944b9f,
// #        the first five calls will generate these values:
// #        0x891432f9, 0x4aa1dccc, 0xc54270fa, 0x9885155f, 0xce83d1b8, ...
// # your code is to be written farther down (see comment below).

fn codgen(seed: &mut u32) -> u32 {
    let mut n: i32 = 32;
    let lseed: u32 = seed.clone(); // copy value of seed
    let mut s: u32 = lseed;
    let x: u32;
    let y: u32;

    //n = [count the number of 0's in word "seed"];
    while s != 0 {
        if s & 1u32 == 1u32 {
            n -= 1;
        }
        s >>= 1; // >> is logic shift
    }

    // x := [rotate "seed" left by 30 bits];
    x = lseed << 30 | ((lseed >> 2) & ((1 << (30 + 1)) - 1));

    // y := [shift "seed" right-ARITHMETIC by 6 bits];
    y = ((lseed as i32) >> 6) as u32;

    // seed := x XOR y XOR n;
    *seed = x ^ y ^ (n as u32);

    // RETURN( seed XOR 0x464b713e );
    (*seed) ^ 0x464b713e
}

fn codgen2(seed: &mut u32) -> u32 {
    let mut n: i32 = 32;
    let mut s: u32 = *seed;
    let x: u32;
    let y: u32;

    while s != 0 {
        if s & 1u32 == 1u32 {
            n -= 1;
        }
        s >>= 1;
    }

    x = *seed << 30 | ((*seed >> 2) & ((1 << (30 + 1)) - 1));
    y = ((*seed as i32) >> 6) as u32;
    *seed = x ^ y ^ (n as u32);
    *seed ^ 0x464b713e
}

fn main() {
    // test that codgen works
    let mut seed = ksymbol!("seed");
    let mut seed_copy = seed;
    kassert!(codgen(&mut seed) == codgen2(&mut seed_copy));
    // unsafe {
    //     ptr::read_volatile(&codgen(&mut seed));
    // }
}
